local LDB = LibStub("LibDataBroker-1.1", true)
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale("WTFrames")
local WTFrames = LibStub("AceAddon-3.0"):NewAddon("WTFrames", "AceConsole-3.0")

local backdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		insets = {left = -1.5, right = -1.5, top = -1.5, bottom = -1.5},
}	
local statusBarTexture = [[Interface\Addons\WTFrames\media\statusbar]]

local anchorRef = {}
local initLDB = {}
local InitConfig = {}
local maintankFrame = {}

function InitConfig()
	WTFramesConfig = {
		options = {
			ShowRaidIcons = true,
			FrameWidth = 120,
			FrameHeight = 20,
			HealthHeight = 15,
			IconSize = 16,
			OnlyShowBlizzardTanks = false,
		},
		frame = {
			point = "BOTTOMLEFT",
			x = 50,
			y = 500,
		}
	}	
end

local options = {
	name = L["WTFrames"],
	type = "group",
	args = {
		confdesc = {
			order = 1,
			type = "description",
			name = string.format("Simple oRA3 Maintank Target Frames - Version: %s", GetAddOnMetadata("WTFrames", "Version") or "N/A"),
			width = "full",
			cmdHidden = true
		},
		frameOptions = {
			type = "group",
			name = "Frame Options",
			inline = true,
			args = {	
				lockFrames = {
					order = 2, 
					type = "toggle", 
					width = "full",
					name = L["Lock frames"],
					desc = L["Lock frames"],
					get = function() return WTFrames:IsLocked() end,
					set = function(_,v) 
						if v then 
							WTFrames:Lock() 
						else 
							WTFrames:UnLock()
						end
					end,
				},		
				onlyShowBlizzardTanks = {
					order = 3, 
					type = "toggle", 
					width = "full",
					name = L["Only show Blizzard Maintanks"],
					desc = L["Doesn't show any personal tanks - only Blizzard tanks (oRA3 shield icon)"],
					get = function() return WTFramesConfig.options.OnlyShowBlizzardTanks end,
					set = function(_,v) WTFramesConfig.options.OnlyShowBlizzardTanks = v 
						WTFrames:UpdateMaintankList()
					end,
				},
			}
		}
	}
}

oUF.TagEvents["[exthp]"] = "UNIT_HEALTH"
oUF.Tags["[exthp]"] = function(unit)
	return UnitIsDead(unit) and "Dead" 
	    or UnitIsGhost(unit) and "Ghost" 
	    or not UnitIsConnected(unit) and "Offline" 
		or string.format('%d%%', floor(UnitHealth(unit) / UnitHealthMax(unit) * 100))
end

oUF.TagEvents["[truncname]"] = "UNIT_TARGET"
oUF.Tags["[truncname]"] = function(unit)
	local name = UnitName(unit)
	if (string.len(name) > 12) then
		name = string.format("%s...", name:sub(1, 12))
	end
	
	return name
end

function WTFrames:chatCommand(input) 	
	local cmd = WTFrames:GetArgs(input)
	
	if(cmd == "" or not cmd)then		
		LibStub("AceConfigDialog-3.0"):Open("WTFrames")
	else 
		LibStub("AceConfigCmd-3.0").HandleCommand(WTFrames, "wtframes", "WTFrames", input)
	end
end

function WTFrames:OnInitialize()
	WTFrames:UpdateConfig()
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("WTFrames", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("WTFrames")
	initLDB()
	
	WTFrames:RegisterChatCommand("wtframes", "chatCommand")
	
end
 
function WTFrames:UpdateConfig()
	if not WTFramesConfig then
		InitConfig()
	end

	-- TODO
end
 
function initLDB()
	if LDB then
		local launcher = LibStub("LibDataBroker-1.1"):NewDataObject("WTFrames", {
			type = "launcher",
			label = "WTFrames",
			icon = "Interface\\Icons\\Ability_Warrior_DefensiveStance",
			OnClick = function(self, button)
				if button == "LeftButton" then
					--if InCombatLockdown() then return end
					if  anchorRef:IsVisible() then
						WTFrames:Lock()
					else
						WTFrames:UnLock()
					end
				else	
					for i, button in ipairs(InterfaceOptionsFrameAddOns.buttons) do
						if button.element and button.element.name == "WTFrames" and button.element.collapsed then
							OptionsListButtonToggle_OnClick(button.toggle)
						end
					end									
					if LibStub("AceConfigDialog-3.0").OpenFrames["WTFrames"] then
						LibStub("AceConfigDialog-3.0"):Close("WTFrames")						
					else
						LibStub("AceConfigDialog-3.0"):Open("WTFrames")
					end	
				end
			end,
			OnTooltipShow = function(tooltip)
				if not tooltip or not tooltip.AddLine then return end
				tooltip:AddLine("Who's tanking - Frames")
				tooltip:AddLine("|cffffff00" .. L["Left-click to toggle anchor."] .. "|r")
				tooltip:AddLine("|cffffff00" .. L["Right-click for options."] .. "|r")
			end
		})		
	end
end

function WTFrames:IsLocked() 
	return not anchorRef:IsVisible()
end

function WTFrames:Lock()
	if not WTFrames:IsLocked() then
		WTFrames:Print(L["Maintank frames are now locked"])
		anchorRef:Hide()
	end
end

function WTFrames:UnLock()
	if WTFrames:IsLocked() then
		WTFrames:Print(L["Maintank frames are now unlocked"])
		anchorRef:Show()
	end
end

function WTFrames:Reset()	
	InitConfig()
end


function oRA3MaintankStyle(self, unit)
	local frameheight = WTFramesConfig.options.FrameHeight
	local framewidth = WTFramesConfig.options.FrameWidth
	local healthheight = WTFramesConfig.options.HealthHeight	

    self:SetAttribute('initial-height', frameheight)
    self:SetAttribute('initial-width', framewidth)      
	
	local hp = CreateFrame('StatusBar', "HealthBar", self) 
    hp:SetPoint("TOPLEFT") 
    hp:SetStatusBarTexture(statusBarTexture) 
	hp:SetWidth(framewidth)
    hp:SetHeight(healthheight) 
    hp:SetPoint("RIGHT") 
    hp:SetPoint("LEFT")
	
	
    hp.bg = hp:CreateTexture(nil, "BORDER")
    hp.bg:SetAllPoints(hp)
    hp.bg:SetTexture(statusBarTexture)
   	hp.bg.multiplier = 0.3
	
	hp.colorReaction = true
	hp.colorClass = true
	hp.colorClassNPC = false
	hp.colorDisconnected = true
	hp.colorTapping = true	
 	hp.frequentUpdates = true
	hp.colorSmooth = true	
	
	local textHP = hp:CreateFontString(nil, 'OVERLAY')
	textHP:SetFontObject(GameFontNormalSmall)
	textHP:SetShadowColor(0,0,0)
	textHP:SetShadowOffset(0.8, -0.8)
	textHP:SetTextColor(1,1,1)
	textHP:SetJustifyH("LEFT")	
	textHP:SetPoint('RIGHT',self,'RIGHT',-2,2)
	textHP.frequentUpdates = true	   	
    self:Tag(textHP,'[exthp]')				
			
	local textName = hp:CreateFontString(nil, 'OVERLAY')
	textName:SetFontObject(GameFontNormalSmall)
	textName:SetShadowColor(0,0,0)
	textName:SetShadowOffset(0.8, -0.8)
	textName:SetTextColor(1,1,1)
	textName:SetJustifyH("LEFT")	
	textName:SetPoint('LEFT',self,'LEFT',17,2)
	textName.frequentUpdates = true	   	
    self:Tag(textName,'[truncname]')
	

	local pp = CreateFrame('StatusBar', nil, self)
	pp:SetHeight(frameheight - healthheight - 1)
	pp:SetStatusBarTexture(statusBarTexture)
	pp:SetPoint("LEFT")
	pp:SetPoint("RIGHT")
	pp:SetPoint("TOP", hp, "BOTTOM", 0, 0)

	pp.colorDisconnected = true
	pp.colorPower = true
	pp.frequentUpdates = true
	pp.colorSmooth = true
	
	pp.bg = pp:CreateTexture(nil, "BORDER")
	pp.bg:SetAllPoints(pp)
	pp.bg:SetTexture(statusBarTexture)
	pp.bg:SetAlpha(.5)
	pp.bg.multiplier = 0.3

	self.Health = hp   	
	self.Power = pp	
		  
    if(WTFramesConfig.options.ShowRaidIcons)then
        local raidIcon = hp:CreateTexture(nil, "OVERLAY")
        raidIcon:SetPoint("LEFT", self, 0, 2)
        raidIcon:SetHeight(WTFramesConfig.options.IconSize)
        raidIcon:SetWidth(WTFramesConfig.options.IconSize)
        raidIcon.frequentUpdates = 0.1		
		self.RaidIcon = raidIcon
    end
	
	
 end
 
 
function WTFrames:GetMaintankList(onlyShowBlizzardTanks)
	local oRA3Tanks = oRA3:GetSortedTanks()	
	
	local res = {}
	
	if onlyShowBlizzardTanks then
		local blizzTanks = oRA3:GetBlizzardTanks()
	
		for k in pairs(oRA3Tanks) do 
			local tank = oRA3Tanks[k]
		
			for kb in pairs(blizzTanks) do
				if tank == blizzTanks[kb] then
					table.insert(res, tank)
				end
			end
		end
	else
		res = oRA3Tanks
	end
	
	return res
end

function WTFrames:UpdateMaintankList()	
	local tanks = WTFrames:GetMaintankList(WTFramesConfig.options.OnlyShowBlizzardTanks)

	maintankFrame:SetAttribute("nameList", table.concat(tanks, ","))
end
 
function WTFrames:OnEnable()	
	oUF:RegisterStyle('oRA3MaintankStyle', oRA3MaintankStyle) 
	oUF:SetActiveStyle("oRA3MaintankStyle")
	
	local oRA3Tanks = oRA3:GetSortedTanks()
	
	local anchor = CreateFrame("Frame", nil, UIParent)
	local anchorText = anchor:CreateFontString(nil, nil, "GameFontNormalSmall")
	anchorText:SetPoint("TOPLEFT")
	anchorText:SetText("WTFrames Anchor")
	anchor:SetWidth(WTFramesConfig.options.FrameWidth)
	--anchor:SetHeight(24 + (#oRA3Tanks * (WTFramesConfig.options.FrameHeight + 2.9)))
	anchor:SetHeight(12)

	--anchor:SetBackdrop(backdrop)
	--anchor:SetBackdropColor(0.09, 0.09, 0.19, 0.5)
	--anchor:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)		
	
	anchor:SetPoint(WTFramesConfig.frame.point, WTFramesConfig.frame.x, WTFramesConfig.frame.y )
	
	anchor:EnableMouse(true)
	anchor:SetMovable(true)
	anchor:SetScript("OnMouseDown", anchor.StartMoving)
	anchor:SetScript("OnMouseUp", function(self, button)
		self:StopMovingOrSizing()
		WTFramesConfig.frame.point, WTFramesConfig.frame.x, WTFramesConfig.frame.y = "BOTTOMLEFT", self:GetLeft(), self:GetBottom()
	end)
	
	anchor:Hide()
			
	anchorRef = anchor
	
	local maintanks = oUF:Spawn("header", "WTFrames_MainTanks", nil, "MTT")
	maintanks:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, 0)
	maintanks:SetManyAttributes(
		"yOffset", 2.9,
		"nameList", table.concat(oRA3Tanks, ","),
		"template", "WTFrames_Maintank",
		"showRaid", true,
		"initial-unitWatch", true,
		"point", "BOTTOM",
		"sortDir", "DESC",
		"filter", "MAINTANK"
	)
	local tankhandler = {}
	function tankhandler:OnTanksUpdated(event, tanks) 
		WTFrames:UpdateMaintankList()		
	end
	function tankhandler:OnTanksChanged(event, tanks) 
		WTFrames:UpdateMaintankList()		
	end	
	
	maintankFrame = maintanks
	
	oRA3.RegisterCallback(tankhandler, "OnTanksUpdated")
	oRA3.RegisterCallback(tankhandler, "OnTanksChanged")	
	
	WTFrames:UpdateMaintankList()
		
	maintanks:Show()
end

